import { __decorate, __awaiter, __generator, __values } from 'tslib';
import { ɵɵdefineInjectable, Injectable, EventEmitter, Input, Output, Component, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { loadGraphModel } from '@tensorflow/tfjs-converter';
import { load, BlazeFaceModel } from '@tensorflow-models/blazeface';
import { CommonModule } from '@angular/common';

var LandmarksService = /** @class */ (function () {
    function LandmarksService() {
        this.$landmarks = new Subject();
    }
    LandmarksService.prototype.setLandmarks = function (landmarks) {
        this.$landmarks.next(landmarks);
    };
    Object.defineProperty(LandmarksService.prototype, "landmarks", {
        get: function () {
            return this.$landmarks.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    LandmarksService.ngInjectableDef = ɵɵdefineInjectable({ factory: function LandmarksService_Factory() { return new LandmarksService(); }, token: LandmarksService, providedIn: "root" });
    LandmarksService = __decorate([
        Injectable({
            providedIn: 'root'
        })
    ], LandmarksService);
    return LandmarksService;
}());

var ResponseAction = /** @class */ (function () {
    function ResponseAction(event) {
        this.event = event;
    }
    return ResponseAction;
}());

var ResponseError = /** @class */ (function () {
    function ResponseError(error, code) {
        this.error = error;
        this.code = code;
    }
    return ResponseError;
}());

var EventAction;
(function (EventAction) {
    EventAction["LEFT"] = "LEFT";
    EventAction["RIGHT"] = "RIGHT";
    EventAction["FRONT"] = "FRONT";
    EventAction["SMILE"] = "SMILE";
})(EventAction || (EventAction = {}));

var EventComplete = /** @class */ (function () {
    function EventComplete(gesture) {
        this.gesture = gesture;
    }
    return EventComplete;
}());

var ErrorType;
(function (ErrorType) {
    ErrorType["WEBGL_EXT_NOT_SUPPORTED"] = "Este navegador no soporta prueba de vida: extensi\u00F3n webgl no soportada";
    ErrorType["GESTURE_NOT_FOUND"] = "Error al validar los gestos seleccionados, gesto no reconocido: ";
    ErrorType["BROWSER_NOT_SUPPORTED"] = "Navegador no soportado";
    ErrorType["NOT_ACCEPT_CAMERA_PERMISSION"] = "No tienes permisos para acceder a la c\u00E1mara";
    ErrorType["MAX_TIME_LIMIT"] = "Tiempo m\u00E1ximo superado";
})(ErrorType || (ErrorType = {}));

var ErrorCode;
(function (ErrorCode) {
    ErrorCode[ErrorCode["WEBGL_EXT_NOT_SUPPORTED"] = -1] = "WEBGL_EXT_NOT_SUPPORTED";
    ErrorCode[ErrorCode["GESTURE_NOT_FOUND"] = -2] = "GESTURE_NOT_FOUND";
    ErrorCode[ErrorCode["BROWSER_NOT_SUPPORTED"] = -3] = "BROWSER_NOT_SUPPORTED";
    ErrorCode[ErrorCode["NOT_ACCEPT_CAMERA_PERMISSION"] = -4] = "NOT_ACCEPT_CAMERA_PERMISSION";
    ErrorCode[ErrorCode["MAX_TIME_LIMIT"] = -5] = "MAX_TIME_LIMIT";
})(ErrorCode || (ErrorCode = {}));

var Point = /** @class */ (function () {
    function Point(x, y) {
        this.x = x;
        this.y = y;
    }
    return Point;
}());

var Landmarks = /** @class */ (function () {
    function Landmarks(eyeRight, eyeLeft, nose, mouth, jawRight, jawLeft) {
        this.eyeRight = new Point(eyeRight[0], eyeRight[1]);
        this.eyeLeft = new Point(eyeLeft[0], eyeLeft[1]);
        this.nose = new Point(nose[0], nose[1]);
        this.mouth = new Point(mouth[0], mouth[1]);
        this.jawRight = new Point(jawRight[0], jawRight[1]);
        this.jawLeft = new Point(jawLeft[0], jawLeft[1]);
    }
    return Landmarks;
}());

var SampleResult = /** @class */ (function () {
    function SampleResult() {
    }
    return SampleResult;
}());

var Mode;
(function (Mode) {
    Mode["NEUTRAL"] = "NEUTRAL";
    Mode["LOW"] = "LOW";
    Mode["MEDIUM"] = "MEDIUM";
    Mode["HIGH"] = "HIGH";
})(Mode || (Mode = {}));

var Side;
(function (Side) {
    Side["LEFT"] = "LEFT";
    Side["RIGHT"] = "RIGHT";
})(Side || (Side = {}));

var POSITION_NEUTRAL = 50;
var THRESHOLD_NEUTRAL_WIDTH = 8;
var THRESHOLD_NEUTRAL_HEIGHT = 8;
var MAX_STD_DEVIATION = 2.5;
var THRESHOLD_MODE = [
    { name: [Mode.NEUTRAL], value: 30 }, { name: [Mode.LOW], value: 45 },
    { name: [Mode.MEDIUM, Mode.LOW], value: 85 }, { name: [Mode.HIGH, Mode.MEDIUM, Mode.LOW], value: 100 }
];
var Utils = /** @class */ (function () {
    function Utils() {
    }
    Utils.validateSample = function (data) {
        var landmarks = this.getLandmarks(data);
        var validateNeutralMouthNose = function () {
            var MIN_THRESHOLD = POSITION_NEUTRAL - THRESHOLD_NEUTRAL_WIDTH;
            var MAX_THRESHOLD = POSITION_NEUTRAL + THRESHOLD_NEUTRAL_WIDTH;
            var boundingBox = Utils.getBoundingBox(data.topLeft, data.bottomRight);
            var proportionMouth = Utils.getProportion(boundingBox, landmarks.mouth);
            var proportionNose = Utils.getProportion(boundingBox, landmarks.nose);
            if (proportionMouth.width >= MIN_THRESHOLD && proportionMouth.width <= MAX_THRESHOLD
                && proportionNose.width >= MIN_THRESHOLD && proportionNose.width <= MAX_THRESHOLD)
                return true;
            else {
                console.log('mouth and nose', proportionMouth, proportionNose);
                return false;
            }
        };
        return this.validateNeutralPosition(landmarks) && validateNeutralMouthNose();
    };
    Utils.analyzeSamples = function (sample) {
        var dataLandmarks = [];
        dataLandmarks[0] = new Array(); // eyeRight
        dataLandmarks[1] = new Array(); // eyeLeft
        dataLandmarks[2] = new Array(); // nose
        dataLandmarks[3] = new Array(); // mouth
        dataLandmarks[4] = new Array(); // jawRight
        dataLandmarks[5] = new Array(); // jawLeft
        var getMean = function (data) {
            var sumWidth = 0;
            var sumHeight = 0;
            data.forEach(function (element) {
                sumWidth += element.width;
                sumHeight += element.height;
            });
            return { width: sumWidth / data.length, height: sumHeight / data.length };
        };
        var getStdDeviation = function (data, mean) {
            var sumWidth = 0;
            var sumHeight = 0;
            data.forEach(function (element) {
                sumWidth += Math.pow(element.width - mean.width, 2);
                sumHeight += Math.pow(element.height - mean.height, 2);
            });
            var stdDeviationWidth = Math.sqrt(sumWidth / (data.length));
            var stdDeviationHeight = Math.sqrt(sumHeight / (data.length));
            return { width: stdDeviationWidth, height: stdDeviationHeight };
        };
        var isValidStdDeviation = function (stdDeviation) {
            if (stdDeviation.width <= MAX_STD_DEVIATION || stdDeviation.height <= MAX_STD_DEVIATION)
                return true;
            else
                return false;
        };
        var getSampleResult = function () {
            var result = new SampleResult();
            dataLandmarks.every(function (element, index) {
                var mean = getMean(element);
                var delta = getStdDeviation(element, mean);
                if (isValidStdDeviation(delta)) {
                    var temp = { width: mean.width, height: mean.height, deltaWidth: delta.width, deltaHeight: delta.height };
                    switch (index) {
                        case 0:
                            result.eyeRight = temp;
                            break;
                        case 1:
                            result.eyeLeft = temp;
                            break;
                        case 2:
                            result.nose = temp;
                            break;
                        case 3:
                            result.mouth = temp;
                            break;
                        case 4:
                            result.jawRight = temp;
                            break;
                        case 5:
                            result.jawLeft = temp;
                            break;
                    }
                    return true;
                }
                else {
                    result = null;
                    return false;
                }
            });
            console.log(result);
            return result;
        };
        sample.forEach(function (element) {
            var boundingBox = Utils.getBoundingBox(element.topLeft, element.bottomRight);
            var landmarks = new Landmarks(element.landmarks[0], element.landmarks[1], element.landmarks[2], // eyeRight, eyeLeft, nose
            element.landmarks[3], element.landmarks[4], element.landmarks[5] // mouth, jawRight, jawLeft
            );
            dataLandmarks[0].push(Utils.getProportion(boundingBox, landmarks.eyeRight));
            dataLandmarks[1].push(Utils.getProportion(boundingBox, landmarks.eyeLeft));
            dataLandmarks[2].push(Utils.getProportion(boundingBox, landmarks.nose));
            dataLandmarks[3].push(Utils.getProportion(boundingBox, landmarks.mouth));
            dataLandmarks[4].push(Utils.getProportion(boundingBox, landmarks.jawRight));
            dataLandmarks[5].push(Utils.getProportion(boundingBox, landmarks.jawLeft));
        });
        return getSampleResult();
    };
    Utils.validateNeutralFace = function (data, sample) {
        var landmarks = this.getLandmarks(data);
        var validateLandmark = function (landmark, sampleLandmark) {
            var value = Utils.getProportion(boundingBox, landmark);
            var minWidth = sampleLandmark.width - (sampleLandmark.deltaWidth + THRESHOLD_NEUTRAL_WIDTH);
            var maxWidth = sampleLandmark.width + (sampleLandmark.deltaWidth + THRESHOLD_NEUTRAL_WIDTH);
            var minHeight = sampleLandmark.height - (sampleLandmark.deltaHeight + THRESHOLD_NEUTRAL_HEIGHT);
            var maxHeight = sampleLandmark.height + (sampleLandmark.deltaHeight + THRESHOLD_NEUTRAL_HEIGHT);
            if (value.width >= minWidth && value.width <= maxWidth
                && value.height >= minHeight && value.height <= maxHeight)
                return true;
            else
                return false;
        };
        var boundingBox = Utils.getBoundingBox(data.topLeft, data.bottomRight);
        if (this.validateNeutralPosition(landmarks) && validateLandmark(landmarks.mouth, sample.mouth) && validateLandmark(landmarks.nose, sample.nose))
            return true;
        else
            return false;
    };
    Utils.validateTurn = function (data, sample, mode, side) {
        var landmarks = this.getLandmarks(data);
        var getProportionNose = function () {
            var boundingBox = Utils.getBoundingBox(data.topLeft, data.bottomRight);
            return Utils.getProportion(boundingBox, landmarks.nose);
        };
        var getSide = function () {
            if (nose.width > sample.nose.width)
                return Side.LEFT;
            else
                return Side.RIGHT;
        };
        var getEquivalence = function () {
            if (side === Side.LEFT) {
                var total = 100 - sample.nose.width;
                var noseProportional = nose.width - sample.nose.width;
                return noseProportional * 100 / total;
            }
            else {
                var noseProportional = nose.width * 100 / sample.nose.width;
                return 100 - noseProportional;
            }
        };
        var validateMode = function () {
            var equivalence = getEquivalence();
            var currentMode = THRESHOLD_MODE.find(function (element) { return equivalence < element.value; });
            if (currentMode && currentMode.name.includes(mode)) {
                console.log('--------------------------------- ');
                console.log(currentMode.name);
                console.log('--------------------------------- ');
                return true;
            }
            else {
                if (currentMode && currentMode.name && currentMode.name[0] !== Mode.NEUTRAL) {
                    console.log('mode incorrect: ' + currentMode.name[0]);
                }
                return false;
            }
        };
        var nose = getProportionNose();
        var boundingBox = Utils.getBoundingBox(data.topLeft, data.bottomRight);
        if (side !== getSide()) {
            console.log('side incorrect: ' + getSide());
        }
        if (side === getSide() && validateMode() && this.validateFaceProportionHeigth(landmarks, boundingBox, sample))
            return true;
        else
            return false;
    };
    Utils.validateFaceProportionHeigth = function (landmarks, boundingBox, sample) {
        var validateLandmark = function (landmark, sampleLandmark, factor) {
            if (factor === void 0) { factor = 2; }
            var value = Utils.getProportion(boundingBox, landmark);
            var minHeight = sampleLandmark.height - (sampleLandmark.deltaHeight + THRESHOLD_NEUTRAL_HEIGHT + factor);
            var maxHeight = sampleLandmark.height + (sampleLandmark.deltaHeight + THRESHOLD_NEUTRAL_HEIGHT + factor);
            if (value.height >= minHeight && value.height <= maxHeight)
                return true;
            else {
                if (value.height <= minHeight) {
                    console.log('incorrect min  h: ' + value.height + ' - min h:' + minHeight + ' error: ' + (minHeight - value.height));
                }
                if (value.height >= maxHeight) {
                    console.log('incorrect max h: ' + value.height + ' - max h: ' + maxHeight + ' error: ' + (value.height - maxHeight));
                }
                return false;
            }
        };
        var validMouth = validateLandmark(landmarks.mouth, sample.mouth, 2);
        var validNose = validateLandmark(landmarks.nose, sample.nose, 3);
        var validEyeLeft = validateLandmark(landmarks.eyeLeft, sample.eyeLeft, 2);
        var validEyeRight = validateLandmark(landmarks.eyeRight, sample.eyeRight, 2);
        var validJawLeft = validateLandmark(landmarks.jawLeft, sample.jawLeft, 2);
        var validJawRight = validateLandmark(landmarks.jawRight, sample.jawRight, 2);
        if (validMouth && validNose && validEyeLeft && validEyeRight && validJawLeft && validJawRight)
            return true;
        else {
            console.log('validMouth: ' + validMouth + '- validNose: ' + validNose + '- validEyeLeft: ' + validEyeLeft
                + '- validEyeRight:  ' + validEyeRight + '-  validJawLeft: ' + validJawLeft + ' - validJawRight: ' + validJawRight);
            return false;
        }
    };
    Utils.getLandmarks = function (data) {
        data.landmarks.forEach(function (landmark) {
            landmark[0] = landmark[0] - data.topLeft[0];
            landmark[1] = landmark[1] - data.topLeft[1];
        });
        var landmarks = new Landmarks(data.landmarks[0], data.landmarks[1], data.landmarks[2], // eyeRight, eyeLeft, nose
        data.landmarks[3], data.landmarks[4], data.landmarks[5] // mouth, jawRight, jawLeft
        );
        return landmarks;
    };
    Utils.getBoundingBox = function (topLeft, bottomRight) {
        return { width: bottomRight[0] - topLeft[0], height: bottomRight[1] - topLeft[1] };
    };
    Utils.validateNeutralPosition = function (landmarks) {
        var validateNeutralEyes = function () {
            if (landmarks.jawLeft.x > landmarks.eyeLeft.x && landmarks.eyeLeft.x > landmarks.eyeRight.x && landmarks.eyeRight.x > landmarks.jawRight.x)
                return true;
            else
                return false;
        };
        var validateNeutralMouth = function () {
            if (landmarks.mouth.x < landmarks.eyeLeft.x && landmarks.mouth.x > landmarks.eyeRight.x)
                return true;
            else
                return false;
        };
        var validateNeutralNose = function () {
            if (landmarks.nose.x < landmarks.eyeLeft.x && landmarks.nose.x > landmarks.eyeRight.x)
                return true;
            else
                return false;
        };
        var validateNeutralMouthNose = function () {
            if (landmarks.mouth.y > landmarks.nose.y)
                return true;
            else
                return false;
        };
        if (validateNeutralEyes() && validateNeutralMouth() && validateNeutralNose() && validateNeutralMouthNose())
            return true;
        else {
            // console.log('incorrec neutral posotion:' +  ' eyes: ' + validateNeutralEyes() + ', mouth: ' +  validateNeutralMouth() + ', nose: ' + validateNeutralNose() + ', mouth and nose: ' + validateNeutralMouthNose());
            return false;
        }
    };
    Utils.getProportion = function (boundingBox, point) {
        var proportionWidth = (point.x * 100) / boundingBox.width;
        var proportionHeight = (point.y * 100) / boundingBox.height;
        return { width: proportionWidth, height: proportionHeight };
    };
    Utils.getImage = function () {
        var image = new Image();
        image.src = './assets/test.png';
        return image;
    };
    return Utils;
}());

var GESTURES = ['LEFT', 'RIGHT', 'SMILE'];
var NgNaatLivenessComponent = /** @class */ (function () {
    // image = Utils.getImage();
    function NgNaatLivenessComponent(landmarksService) {
        var _this = this;
        this.landmarksService = landmarksService;
        this.videoMirror = true;
        this.sampleSize = 25;
        this.mode = Mode.MEDIUM;
        this.onerror = new EventEmitter();
        this.detectionReady = new EventEmitter();
        this.calibrationPercentage = new EventEmitter();
        this.calibrationReady = new EventEmitter();
        this.faceDetected = new EventEmitter();
        // @Input() fps = 1;
        this.events = [];
        // face detected
        // faceDetectedInterval: NodeJS.Timer;
        this.isFaceDetected = false;
        // process complete
        this.isProcessComplete = false;
        // samples face
        this.sampleData = [];
        this.actionComplete = new EventEmitter();
        this.gestureComplete = new EventEmitter();
        this.processComplete = new EventEmitter();
        this.getSampleData = function () { return __awaiter(_this, void 0, void 0, function () {
            var faces;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.model.estimateFaces(this.video)];
                    case 1:
                        faces = _a.sent();
                        if (faces.length > 0 && Utils.validateSample(faces[0])) {
                            this.sampleData.push(faces[0]);
                            this.calibrationPercentage.next(Math.round(this.sampleData.length * 100 / this.sampleSize));
                        }
                        if (this.sampleData.length === this.sampleSize) { // validate size of samples
                            this.sampleResult = Utils.analyzeSamples(this.sampleData); // analyze samples
                            if (this.sampleResult) { // validate integrity of the samples
                                this.calibrationReady.next();
                                this.initEvents();
                            }
                            else {
                                console.log('get sample again...');
                                this.sampleData = [];
                                this.getSampleData();
                            }
                        }
                        else {
                            requestAnimationFrame(this.getSampleData);
                        }
                        return [2 /*return*/];
                }
            });
        }); };
        this.detectFace = function () { return __awaiter(_this, void 0, void 0, function () {
            var fechaInicio, faces, fechaFin;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        fechaInicio = new Date().getTime();
                        return [4 /*yield*/, this.model.estimateFaces(this.video, false, false)];
                    case 1:
                        faces = _a.sent();
                        fechaFin = new Date().getTime();
                        console.log('time: ' + (fechaFin - fechaInicio));
                        if (faces.length > 0) {
                            this.faceDetected.next(true);
                            this.isFaceDetected = true;
                            if (this.drawFace)
                                this.draw(faces[0]);
                            if (this.activeEmiterEvent)
                                this.landmarksService.setLandmarks(faces[0]);
                        }
                        else {
                            this.isFaceDetected = false;
                            this.faceDetected.next(false);
                        }
                        if (!this.isProcessComplete)
                            requestAnimationFrame(this.detectFace);
                        return [2 /*return*/];
                }
            });
        }); };
    }
    NgNaatLivenessComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.initElements();
        Promise.all([this.validateWebgl(), this.validateGestures(), this.getUserMedia(), this.loadModel()]).then(function (values) {
            _this.stream = values[2]; // getUserMedia
            _this.model = values[3]; // loadModel
            _this.video.srcObject = _this.stream;
            _this.gestureComplete.subscribe(function (res) {
                _this.validateActionComplete(res.event);
            });
        }).catch(function (error) { return _this.onerror.emit(error); });
    };
    NgNaatLivenessComponent.prototype.initElements = function () {
        var _this = this;
        this.video = document.getElementById('video');
        this.canvas = document.getElementById('overlay');
        this.canvasContext = this.canvas.getContext('2d');
        this.video.onloadeddata = function () { return _this.videoLoaded(); };
    };
    NgNaatLivenessComponent.prototype.validateWebgl = function () {
        return new Promise(function (resolve, reject) {
            var canvas = document.createElement('canvas');
            var gl = canvas.getContext('webgl');
            if (gl.getExtension('WEBGL_depth_texture'))
                resolve();
            else
                reject(new ResponseError(ErrorType.WEBGL_EXT_NOT_SUPPORTED, ErrorCode.WEBGL_EXT_NOT_SUPPORTED));
        });
    };
    NgNaatLivenessComponent.prototype.validateGestures = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.gestures.forEach(function (element) {
                _this.events.push(new EventComplete(element.toUpperCase()));
                if (!GESTURES.includes(element.toUpperCase()))
                    reject(new ResponseError(ErrorType.GESTURE_NOT_FOUND + element, ErrorCode.GESTURE_NOT_FOUND));
            });
            resolve();
        });
    };
    NgNaatLivenessComponent.prototype.getUserMedia = function () {
        return new Promise(function (resolve, reject) {
            if (navigator.mediaDevices && navigator.mediaDevices.getUserMedia) {
                navigator.mediaDevices.getUserMedia({ video: true }).then(function (stream) { return resolve(stream); })
                    .catch(function () { return reject(new ResponseError(ErrorType.NOT_ACCEPT_CAMERA_PERMISSION, ErrorCode.NOT_ACCEPT_CAMERA_PERMISSION)); });
            }
            else
                reject(new ResponseError(ErrorType.BROWSER_NOT_SUPPORTED, ErrorCode.BROWSER_NOT_SUPPORTED));
        });
    };
    NgNaatLivenessComponent.prototype.loadModel = function (_a) {
        var _b = _a === void 0 ? {} : _a, _c = _b.maxFaces, maxFaces = _c === void 0 ? 1 : _c, _d = _b.inputWidth, inputWidth = _d === void 0 ? 128 : _d, _e = _b.inputHeight, inputHeight = _e === void 0 ? 128 : _e, _f = _b.iouThreshold, iouThreshold = _f === void 0 ? 0.7 : _f, _g = _b.scoreThreshold, scoreThreshold = _g === void 0 ? 0.999 : _g;
        return __awaiter(this, void 0, void 0, function () {
            var blazefaceLoaded, model;
            return __generator(this, function (_h) {
                switch (_h.label) {
                    case 0:
                        if (!this.modelPath) return [3 /*break*/, 2];
                        return [4 /*yield*/, loadGraphModel(this.modelPath)];
                    case 1:
                        blazefaceLoaded = _h.sent();
                        model = new BlazeFaceModel(blazefaceLoaded, inputWidth, inputHeight, maxFaces, iouThreshold, scoreThreshold);
                        return [2 /*return*/, model];
                    case 2: return [2 /*return*/, load({ maxFaces: 1, iouThreshold: 0.7, scoreThreshold: 0.999 })];
                }
            });
        });
    };
    NgNaatLivenessComponent.prototype.videoLoaded = function () {
        this.canvas.width = this.video.videoWidth;
        this.canvas.height = this.video.videoHeight;
        this.firstDetection();
    };
    NgNaatLivenessComponent.prototype.firstDetection = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.model.estimateFaces(this.video)];
                    case 1:
                        _a.sent();
                        this.detectionReady.next();
                        console.log('get sample...');
                        this.getSampleData();
                        return [2 /*return*/];
                }
            });
        });
    };
    NgNaatLivenessComponent.prototype.initEvents = function () {
        // this.ngZone.runOutsideAngular( () => this.faceDetectedInterval =  setInterval(() => this.detectFace(), 1000 / this.fps));
        // this.ngZone.runOutsideAngular( () => this.detectFace());
        console.log('init events');
        this.detectFace();
        this.listenEvents();
        if (this.timer)
            this.startTimer();
    };
    NgNaatLivenessComponent.prototype.draw = function (predictions) {
        var e_1, _a;
        this.canvasContext.clearRect(0, 0, this.canvas.width, this.canvas.height);
        var start = predictions.topLeft;
        var end = predictions.bottomRight;
        var size = [end[0] - start[0], end[1] - start[1]];
        this.canvasContext.fillStyle = 'rgba(255, 255, 255, 0.5)';
        this.canvasContext.fillRect(start[0], start[1], size[0], size[1]);
        var landmarks = predictions.landmarks;
        this.canvasContext.fillStyle = 'blue';
        var count = 0;
        try {
            for (var landmarks_1 = __values(landmarks), landmarks_1_1 = landmarks_1.next(); !landmarks_1_1.done; landmarks_1_1 = landmarks_1.next()) {
                var landmark = landmarks_1_1.value;
                var x = landmark[0];
                var y = landmark[1];
                this.canvasContext.fillRect(x, y, 5, 5);
                // this.canvasContext.fillText(`${count}`, x, y);
                count++;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (landmarks_1_1 && !landmarks_1_1.done && (_a = landmarks_1.return)) _a.call(landmarks_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    NgNaatLivenessComponent.prototype.listenEvents = function () {
        var _this = this;
        if (this.events.filter(function (element) { return element.complete; }).length === this.gestures.length) {
            this.video.pause();
            this.getCapture().then(function (res) {
                _this.processComplete.next(res);
                _this.ngOnDestroy();
            });
        }
        else {
            var currentEvent = this.events.find(function (element) { return !element.complete; });
            this.executeEvent(currentEvent.gesture);
        }
    };
    NgNaatLivenessComponent.prototype.validateActionComplete = function (event) {
        var currentEvent = this.events.find(function (element) { return !element.complete; });
        if (event === currentEvent.gesture) {
            currentEvent.complete = true;
            this.listenEvents();
        }
    };
    NgNaatLivenessComponent.prototype.executeEvent = function (gesture) {
        if (gesture === 'LEFT')
            this.listenTurnLeft();
        else if (gesture === 'RIGHT')
            this.listenTurnRight();
        // else if (gesture === 'SMILE') this.listenSmile();
    };
    NgNaatLivenessComponent.prototype.listenTurnLeft = function () {
        var _this = this;
        this.activeEmiterEvent = true;
        this.pointsSubscriptionLeft = this.landmarksService.landmarks.subscribe(function (response) {
            if (_this.isFaceDetected && Utils.validateTurn(response, _this.sampleResult, _this.mode, Side.LEFT)) {
                _this.actionComplete.next(new ResponseAction(EventAction.LEFT));
                _this.pointsSubscriptionLeft.unsubscribe();
                _this.listenFront(true);
            }
        });
    };
    NgNaatLivenessComponent.prototype.listenTurnRight = function () {
        var _this = this;
        this.activeEmiterEvent = true;
        this.pointsSubscriptionRight = this.landmarksService.landmarks.subscribe(function (response) {
            if (_this.isFaceDetected && Utils.validateTurn(response, _this.sampleResult, _this.mode, Side.RIGHT)) {
                _this.actionComplete.next(new ResponseAction(EventAction.RIGHT));
                _this.pointsSubscriptionRight.unsubscribe();
                _this.listenFront(false);
            }
        });
    };
    NgNaatLivenessComponent.prototype.listenFront = function (isLeft) {
        var _this = this;
        this.pointsSubscriptionFront = this.landmarksService.landmarks.subscribe(function (response) {
            if (_this.isFaceDetected && Utils.validateNeutralFace(response, _this.sampleResult)) {
                _this.activeEmiterEvent = false;
                _this.pointsSubscriptionFront.unsubscribe();
                _this.actionComplete.next(new ResponseAction(EventAction.FRONT));
                _this.gestureComplete.next(new ResponseAction(isLeft ? EventAction.LEFT : EventAction.RIGHT));
            }
        });
    };
    NgNaatLivenessComponent.prototype.listenSmile = function () {
    };
    NgNaatLivenessComponent.prototype.getCapture = function () {
        var _this = this;
        // TODO: validate image before send
        return new Promise(function (resolve, reject) {
            try {
                var canvas = document.createElement('canvas');
                var ctx = canvas.getContext('2d');
                canvas.width = _this.video.videoWidth;
                canvas.height = _this.video.videoHeight;
                ctx.drawImage(_this.video, 0, 0);
                resolve(canvas.toDataURL());
            }
            catch (error) {
                reject(error);
            }
        });
    };
    NgNaatLivenessComponent.prototype.startTimer = function () {
        var _this = this;
        this.timerTimeout = setTimeout(function () {
            _this.ngOnDestroy();
            _this.onerror.next(new ResponseError(ErrorType.MAX_TIME_LIMIT, ErrorCode.MAX_TIME_LIMIT));
        }, this.timer * 1000);
    };
    NgNaatLivenessComponent.prototype.cameraOff = function () {
        if (this.stream) {
            this.stream.getTracks().forEach(function (track) {
                track.stop();
            });
        }
    };
    NgNaatLivenessComponent.prototype.ngOnDestroy = function () {
        this.cameraOff();
        if (this.timerTimeout)
            clearTimeout(this.timerTimeout);
        this.isProcessComplete = true;
        // if (this.faceDetectedInterval) clearInterval(this.faceDetectedInterval);
        if (this.pointsSubscriptionLeft)
            this.pointsSubscriptionLeft.unsubscribe();
        if (this.pointsSubscriptionRight)
            this.pointsSubscriptionRight.unsubscribe();
        if (this.pointsSubscriptionFront)
            this.pointsSubscriptionFront.unsubscribe();
    };
    NgNaatLivenessComponent.ctorParameters = function () { return [
        { type: LandmarksService }
    ]; };
    __decorate([
        Input()
    ], NgNaatLivenessComponent.prototype, "drawFace", void 0);
    __decorate([
        Input()
    ], NgNaatLivenessComponent.prototype, "videoMirror", void 0);
    __decorate([
        Input()
    ], NgNaatLivenessComponent.prototype, "gestures", void 0);
    __decorate([
        Input()
    ], NgNaatLivenessComponent.prototype, "timer", void 0);
    __decorate([
        Input()
    ], NgNaatLivenessComponent.prototype, "sampleSize", void 0);
    __decorate([
        Input()
    ], NgNaatLivenessComponent.prototype, "mode", void 0);
    __decorate([
        Input()
    ], NgNaatLivenessComponent.prototype, "modelPath", void 0);
    __decorate([
        Output()
    ], NgNaatLivenessComponent.prototype, "onerror", void 0);
    __decorate([
        Output()
    ], NgNaatLivenessComponent.prototype, "detectionReady", void 0);
    __decorate([
        Output()
    ], NgNaatLivenessComponent.prototype, "calibrationPercentage", void 0);
    __decorate([
        Output()
    ], NgNaatLivenessComponent.prototype, "calibrationReady", void 0);
    __decorate([
        Output()
    ], NgNaatLivenessComponent.prototype, "faceDetected", void 0);
    __decorate([
        Output()
    ], NgNaatLivenessComponent.prototype, "actionComplete", void 0);
    __decorate([
        Output()
    ], NgNaatLivenessComponent.prototype, "gestureComplete", void 0);
    __decorate([
        Output()
    ], NgNaatLivenessComponent.prototype, "processComplete", void 0);
    NgNaatLivenessComponent = __decorate([
        Component({
            selector: 'ng-naat-liveness',
            template: "<ng-content select=\"[body]\"></ng-content>\n{{events | json}}\n<div class=\"container-liveness\" [ngClass]=\"{'videoRotate': videoMirror}\">\n  <video  id=\"video\" autoplay muted playsinline></video>\n  <canvas id=\"overlay\"></canvas>\n</div> ",
            styles: [".container-liveness{display:flex;justify-content:center;align-items:center;width:100%;height:100%;flex-direction:column}#video{max-width:100%}#overlay{position:fixed}.videoRotate{transform:rotateY(180deg)}"]
        })
    ], NgNaatLivenessComponent);
    return NgNaatLivenessComponent;
}());

var GESTURES$1 = ['LEFT', 'RIGHT', 'SMILE'];
var NgNaatLivenessWorkersComponent = /** @class */ (function () {
    // image = Utils.getImage();
    function NgNaatLivenessWorkersComponent(landmarksService) {
        var _this = this;
        this.landmarksService = landmarksService;
        this.videoMirror = true;
        this.sampleSize = 25;
        this.workerPath = './assets/liveness/worker/liveness.worker.js';
        this.mode = Mode.MEDIUM;
        this.onerror = new EventEmitter();
        this.detectionReady = new EventEmitter();
        this.calibrationPercentage = new EventEmitter();
        this.calibrationReady = new EventEmitter();
        this.faceDetected = new EventEmitter();
        this.events = [];
        // face detected
        // faceDetectedInterval: NodeJS.Timer;
        this.isFaceDetected = false;
        // process complete
        this.isProcessComplete = false;
        // samples face
        this.sampleData = [];
        this.actionComplete = new EventEmitter();
        this.gestureComplete = new EventEmitter();
        this.processComplete = new EventEmitter();
        this.workerReady = new Subject();
        this.workerDtetectedFace = new Subject();
        this.detectFaceWorker = function () {
            // this.worker.postMessage(data);
        };
        this.getSampleData = function () { return __awaiter(_this, void 0, void 0, function () {
            var faces;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.estimateFaces()];
                    case 1:
                        faces = _a.sent();
                        if (faces.length > 0 && Utils.validateSample(faces[0])) {
                            this.sampleData.push(faces[0]);
                            this.calibrationPercentage.next(Math.round(this.sampleData.length * 100 / this.sampleSize));
                        }
                        if (this.sampleData.length === this.sampleSize) { // validate size of samples
                            this.sampleResult = Utils.analyzeSamples(this.sampleData); // analyze samples
                            if (this.sampleResult) { // validate integrity of the samples
                                this.calibrationReady.next();
                                this.initEvents();
                            }
                            else {
                                console.log('get sample again...');
                                this.sampleData = [];
                                this.getSampleData();
                            }
                        }
                        else {
                            requestAnimationFrame(this.getSampleData);
                        }
                        return [2 /*return*/];
                }
            });
        }); };
        this.detectFace = function () { return __awaiter(_this, void 0, void 0, function () {
            var faces;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.estimateFaces()];
                    case 1:
                        faces = _a.sent();
                        if (faces.length > 0) {
                            this.faceDetected.next(true);
                            this.isFaceDetected = true;
                            if (this.drawFace)
                                this.draw(faces[0]);
                            if (this.activeEmiterEvent)
                                this.landmarksService.setLandmarks(faces[0]);
                        }
                        else {
                            this.isFaceDetected = false;
                            this.faceDetected.next(false);
                        }
                        if (!this.isProcessComplete)
                            requestAnimationFrame(this.detectFace);
                        return [2 /*return*/];
                }
            });
        }); };
        console.log('WORKERS VERSION 3.0');
    }
    NgNaatLivenessWorkersComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.initElements();
        Promise.all([this.validateGestures(), this.getUserMedia(), this.loadWorker()]).then(function (values) {
            _this.stream = values[1]; // getUserMedia
            _this.video.srcObject = _this.stream;
            _this.gestureComplete.subscribe(function (res) {
                _this.validateActionComplete(res.event);
            });
        }).catch(function (error) { return _this.onerror.emit(error); });
    };
    NgNaatLivenessWorkersComponent.prototype.initElements = function () {
        var _this = this;
        this.video = document.getElementById('video');
        this.canvas = document.getElementById('overlay');
        this.canvasWorker = document.createElement('canvas');
        this.canvasContext = this.canvas.getContext('2d');
        this.canvasContextWorker = this.canvasWorker.getContext('2d');
        this.video.onloadeddata = function () { return _this.videoLoaded(); };
    };
    NgNaatLivenessWorkersComponent.prototype.validateGestures = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.gestures.forEach(function (element) {
                _this.events.push(new EventComplete(element.toUpperCase()));
                if (!GESTURES$1.includes(element.toUpperCase()))
                    reject(new ResponseError(ErrorType.GESTURE_NOT_FOUND + element, ErrorCode.GESTURE_NOT_FOUND));
            });
            resolve();
        });
    };
    NgNaatLivenessWorkersComponent.prototype.getUserMedia = function () {
        return new Promise(function (resolve, reject) {
            if (navigator.mediaDevices && navigator.mediaDevices.getUserMedia) {
                navigator.mediaDevices.getUserMedia({ video: true }).then(function (stream) { return resolve(stream); })
                    .catch(function () { return reject(new ResponseError(ErrorType.NOT_ACCEPT_CAMERA_PERMISSION, ErrorCode.NOT_ACCEPT_CAMERA_PERMISSION)); });
            }
            else
                reject(new ResponseError(ErrorType.BROWSER_NOT_SUPPORTED, ErrorCode.BROWSER_NOT_SUPPORTED));
        });
    };
    NgNaatLivenessWorkersComponent.prototype.loadWorker = function () {
        var _this = this;
        return new Promise(function (resolve) {
            _this.initWorker();
            _this.workerReady.subscribe(function () { return resolve(); });
        });
    };
    NgNaatLivenessWorkersComponent.prototype.initWorker = function () {
        var _this = this;
        this.worker = new Worker(this.workerPath);
        this.worker.onmessage = (function (response) {
            if (response.data.event === 'loadModel')
                _this.workerReady.next();
            if (response.data.event === 'detectFace')
                _this.workerDtetectedFace.next(response.data.faces);
        });
        this.worker.postMessage({ event: 'loadModel' });
    };
    NgNaatLivenessWorkersComponent.prototype.videoLoaded = function () {
        this.canvas.width = this.video.videoWidth;
        this.canvas.height = this.video.videoHeight;
        this.canvasWorker.width = this.video.videoWidth / 2;
        this.canvasWorker.height = this.video.videoHeight / 2;
        this.firstDetection();
    };
    NgNaatLivenessWorkersComponent.prototype.firstDetection = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.estimateFaces()];
                    case 1:
                        _a.sent();
                        this.detectionReady.next();
                        console.log('get sample...');
                        this.getSampleData();
                        return [2 /*return*/];
                }
            });
        });
    };
    NgNaatLivenessWorkersComponent.prototype.estimateFaces = function () {
        var _this = this;
        return new Promise(function (resolve) {
            _this.canvasContextWorker.clearRect(0, 0, _this.canvasWorker.width, _this.canvasWorker.height);
            _this.canvasContextWorker.drawImage(_this.video, 0, 0, _this.canvasWorker.width, _this.canvasWorker.height);
            var data = { event: 'detectFace', data: _this.canvasContextWorker.getImageData(0, 0, _this.canvasWorker.width, _this.canvasWorker.height) };
            var subcription = _this.workerDtetectedFace.subscribe(function (faces) {
                subcription.unsubscribe();
                resolve(faces);
            });
            _this.worker.postMessage(data);
        });
    };
    NgNaatLivenessWorkersComponent.prototype.initEvents = function () {
        // this.ngZone.runOutsideAngular( () => this.faceDetectedInterval =  setInterval(() => this.detectFace(), 1000 / this.fps));
        // this.ngZone.runOutsideAngular( () => this.detectFace());
        console.log('init events');
        this.detectFace();
        this.listenEvents();
        if (this.timer)
            this.startTimer();
    };
    NgNaatLivenessWorkersComponent.prototype.draw = function (predictions) {
        var e_1, _a;
        this.canvasContext.clearRect(0, 0, this.canvas.width, this.canvas.height);
        var start = predictions.topLeft;
        var end = predictions.bottomRight;
        var size = [end[0] - start[0], end[1] - start[1]];
        this.canvasContext.fillStyle = 'rgba(255, 255, 255, 0.5)';
        this.canvasContext.fillRect(start[0], start[1], size[0], size[1]);
        var landmarks = predictions.landmarks;
        this.canvasContext.fillStyle = 'blue';
        var count = 0;
        try {
            for (var landmarks_1 = __values(landmarks), landmarks_1_1 = landmarks_1.next(); !landmarks_1_1.done; landmarks_1_1 = landmarks_1.next()) {
                var landmark = landmarks_1_1.value;
                var x = landmark[0];
                var y = landmark[1];
                this.canvasContext.fillRect(x, y, 5, 5);
                // this.canvasContext.fillText(`${count}`, x, y);
                count++;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (landmarks_1_1 && !landmarks_1_1.done && (_a = landmarks_1.return)) _a.call(landmarks_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    NgNaatLivenessWorkersComponent.prototype.listenEvents = function () {
        var _this = this;
        if (this.events.filter(function (element) { return element.complete; }).length === this.gestures.length) {
            this.video.pause();
            this.getCapture().then(function (res) {
                _this.processComplete.next(res);
                _this.ngOnDestroy();
            });
        }
        else {
            var currentEvent = this.events.find(function (element) { return !element.complete; });
            this.executeEvent(currentEvent.gesture);
        }
    };
    NgNaatLivenessWorkersComponent.prototype.validateActionComplete = function (event) {
        var currentEvent = this.events.find(function (element) { return !element.complete; });
        if (event === currentEvent.gesture) {
            currentEvent.complete = true;
            this.listenEvents();
        }
    };
    NgNaatLivenessWorkersComponent.prototype.executeEvent = function (gesture) {
        if (gesture === 'LEFT')
            this.listenTurnLeft();
        else if (gesture === 'RIGHT')
            this.listenTurnRight();
        // else if (gesture === 'SMILE') this.listenSmile();
    };
    NgNaatLivenessWorkersComponent.prototype.listenTurnLeft = function () {
        var _this = this;
        this.activeEmiterEvent = true;
        this.pointsSubscriptionLeft = this.landmarksService.landmarks.subscribe(function (response) {
            if (_this.isFaceDetected && Utils.validateTurn(response, _this.sampleResult, _this.mode, Side.LEFT)) {
                _this.actionComplete.next(new ResponseAction(EventAction.LEFT));
                _this.pointsSubscriptionLeft.unsubscribe();
                _this.listenFront(true);
            }
        });
    };
    NgNaatLivenessWorkersComponent.prototype.listenTurnRight = function () {
        var _this = this;
        this.activeEmiterEvent = true;
        this.pointsSubscriptionRight = this.landmarksService.landmarks.subscribe(function (response) {
            if (_this.isFaceDetected && Utils.validateTurn(response, _this.sampleResult, _this.mode, Side.RIGHT)) {
                _this.actionComplete.next(new ResponseAction(EventAction.RIGHT));
                _this.pointsSubscriptionRight.unsubscribe();
                _this.listenFront(false);
            }
        });
    };
    NgNaatLivenessWorkersComponent.prototype.listenFront = function (isLeft) {
        var _this = this;
        this.pointsSubscriptionFront = this.landmarksService.landmarks.subscribe(function (response) {
            if (_this.isFaceDetected && Utils.validateNeutralFace(response, _this.sampleResult)) {
                _this.activeEmiterEvent = false;
                _this.pointsSubscriptionFront.unsubscribe();
                _this.actionComplete.next(new ResponseAction(EventAction.FRONT));
                _this.gestureComplete.next(new ResponseAction(isLeft ? EventAction.LEFT : EventAction.RIGHT));
            }
        });
    };
    NgNaatLivenessWorkersComponent.prototype.listenSmile = function () {
    };
    NgNaatLivenessWorkersComponent.prototype.getCapture = function () {
        var _this = this;
        // TODO: validate image before send
        return new Promise(function (resolve, reject) {
            try {
                var canvas = document.createElement('canvas');
                var ctx = canvas.getContext('2d');
                canvas.width = _this.video.videoWidth;
                canvas.height = _this.video.videoHeight;
                ctx.drawImage(_this.video, 0, 0);
                resolve(canvas.toDataURL());
            }
            catch (error) {
                reject(error);
            }
        });
    };
    NgNaatLivenessWorkersComponent.prototype.startTimer = function () {
        var _this = this;
        this.timerTimeout = setTimeout(function () {
            _this.ngOnDestroy();
            _this.onerror.next(new ResponseError(ErrorType.MAX_TIME_LIMIT, ErrorCode.MAX_TIME_LIMIT));
        }, this.timer * 1000);
    };
    NgNaatLivenessWorkersComponent.prototype.cameraOff = function () {
        if (this.stream) {
            this.stream.getTracks().forEach(function (track) {
                track.stop();
            });
        }
    };
    NgNaatLivenessWorkersComponent.prototype.ngOnDestroy = function () {
        this.cameraOff();
        if (this.timerTimeout)
            clearTimeout(this.timerTimeout);
        this.isProcessComplete = true;
        if (this.pointsSubscriptionLeft)
            this.pointsSubscriptionLeft.unsubscribe();
        if (this.pointsSubscriptionRight)
            this.pointsSubscriptionRight.unsubscribe();
        if (this.pointsSubscriptionFront)
            this.pointsSubscriptionFront.unsubscribe();
    };
    NgNaatLivenessWorkersComponent.ctorParameters = function () { return [
        { type: LandmarksService }
    ]; };
    __decorate([
        Input()
    ], NgNaatLivenessWorkersComponent.prototype, "drawFace", void 0);
    __decorate([
        Input()
    ], NgNaatLivenessWorkersComponent.prototype, "videoMirror", void 0);
    __decorate([
        Input()
    ], NgNaatLivenessWorkersComponent.prototype, "gestures", void 0);
    __decorate([
        Input()
    ], NgNaatLivenessWorkersComponent.prototype, "timer", void 0);
    __decorate([
        Input()
    ], NgNaatLivenessWorkersComponent.prototype, "sampleSize", void 0);
    __decorate([
        Input()
    ], NgNaatLivenessWorkersComponent.prototype, "workerPath", void 0);
    __decorate([
        Input()
    ], NgNaatLivenessWorkersComponent.prototype, "mode", void 0);
    __decorate([
        Input()
    ], NgNaatLivenessWorkersComponent.prototype, "modelPath", void 0);
    __decorate([
        Output()
    ], NgNaatLivenessWorkersComponent.prototype, "onerror", void 0);
    __decorate([
        Output()
    ], NgNaatLivenessWorkersComponent.prototype, "detectionReady", void 0);
    __decorate([
        Output()
    ], NgNaatLivenessWorkersComponent.prototype, "calibrationPercentage", void 0);
    __decorate([
        Output()
    ], NgNaatLivenessWorkersComponent.prototype, "calibrationReady", void 0);
    __decorate([
        Output()
    ], NgNaatLivenessWorkersComponent.prototype, "faceDetected", void 0);
    __decorate([
        Output()
    ], NgNaatLivenessWorkersComponent.prototype, "actionComplete", void 0);
    __decorate([
        Output()
    ], NgNaatLivenessWorkersComponent.prototype, "gestureComplete", void 0);
    __decorate([
        Output()
    ], NgNaatLivenessWorkersComponent.prototype, "processComplete", void 0);
    NgNaatLivenessWorkersComponent = __decorate([
        Component({
            selector: 'ng-naat-liveness-workers',
            template: "<ng-content select=\"[body]\"></ng-content>\n{{events | json}}\n<div class=\"container-liveness\" [ngClass]=\"{'videoRotate': videoMirror}\">\n  <video  id=\"video\" autoplay muted playsinline></video>\n  <canvas id=\"overlay\"></canvas>\n</div> ",
            styles: [".container-liveness{display:flex;justify-content:center;align-items:center;width:100%;height:100%;flex-direction:column}#video{max-width:100%}#overlay{position:fixed}.videoRotate{transform:rotateY(180deg)}"]
        })
    ], NgNaatLivenessWorkersComponent);
    return NgNaatLivenessWorkersComponent;
}());

var NgNaatLivenessModule = /** @class */ (function () {
    function NgNaatLivenessModule() {
    }
    NgNaatLivenessModule = __decorate([
        NgModule({
            declarations: [NgNaatLivenessComponent, NgNaatLivenessWorkersComponent],
            imports: [
                CommonModule
            ],
            exports: [NgNaatLivenessComponent, NgNaatLivenessWorkersComponent]
        })
    ], NgNaatLivenessModule);
    return NgNaatLivenessModule;
}());

/*
 * Public API Surface of ng-naat-liveness
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgNaatLivenessComponent, NgNaatLivenessModule, NgNaatLivenessWorkersComponent, LandmarksService as ɵa };
//# sourceMappingURL=ng-naat-liveness.js.map
